/*
 * heat2d-e.c -- solve the heat equation in a two dimensional square region
 * by explicit finite difference method.
 */

#include <stdio.h>
#include <math.h>

/* to use matrix, new_matrix() */
#include <matrix.h>

/* to use GLSC */
#define G_DOUBLE
#include <glsc.h>

main()
{
    int             N, i, j, k, skip, Kmax;
    double          h, lambda, tau, Tmax, t, c, dt;
    matrix          u, newu;
    double          f();

    printf("N: "); scanf("%d", &N);
    if ((u = new_matrix(N + 1, N + 1)) == NULL) {
        fprintf(stderr, "配列 u を確保できませんでした。");
        exit(1);
    }
    if ((newu = new_matrix(N + 1, N + 1)) == NULL) {
        fprintf(stderr, "配列 newu を確保できませんでした。");
        exit(1);
    }
    printf("λ: "); scanf("%lf", &lambda);
    printf("Tmax: "); scanf("%lf", &Tmax);

    h = 1.0 / N;
    tau = lambda * h * h;

    printf("τ=%g ですが、Δtをどうしますか: ", tau);
    scanf("%lf",  &dt);
    skip = dt / tau;

    g_init("Meta", 250.0, 160.0);
    g_device(G_BOTH);
    g_def_scale(0, 0.0, 1.0, 0.0, 1.0, 30.0, 70.0, 100.0, 72.0);
    g_def_scale(4, -1.0, 1.0, -1.0, 1.0, 30.0, 30.0, 100.0, 100.0);
    g_def_line(0, G_BLACK, 0, G_LINE_SOLID);
    g_sel_scale(0);

    for (i = 0; i <= N; i++)
        for (j = 0; j <= N; j++)
            u[i][j] = f(i * h, j * h);

    c = 1.0 - 4.0 * lambda;

    g_cls();
    g_hidden2(1.0, 1.0, 0.4, -1.0, 1.0, 5.0, 25.0, 20.0, 20.0, 20.0,
        150.0, 100.0, u, N + 1, N + 1,
        1, G_SIDE_NONE, 2, 1);

    Kmax = (Tmax + 0.1 * tau) / tau;
    for (k = 1; k <= Kmax; k++) {
        for (i = 1; i < N; i++)
            for (j = 1; j < N; j++)
                newu[i][j] = c * u[i][j]
                    + lambda * (u[i + 1][j] + u[i - 1][j]
			      + u[i][j + 1] + u[i][j - 1]);
        for (i = 0; i <= N; i++)
            newu[0][i] = newu[N][i] = newu[i][0] = newu[i][N] = 0.0;
        for (i = 0; i <= N; i++)
            for (j = 0; j <= N; j++)
                u[i][j] = newu[i][j];
        if (k % skip == 0) {
            g_cls();
            g_hidden2(1.0, 1.0, 0.4, -1.0, 1.0, 5.0, 25.0, 20.0, 20.0, 20.0,
                150.0, 100.0, u, N + 1, N + 1,
                1, G_SIDE_NONE, 2, 1);
        }
        t = tau * k;
    }

    /* マウスでクリックされるのを待つ */
    sleep(-1.0);
    /* ウィンドウを消す */
    g_term();
    return 0;
}

double f(x, y)
double x, y;
{
    return 16 * x * (1 - x) * y * (1 - y);
}
