/*
 * matrix.h
 */

#ifndef MATUTIL
#define MATUTIL

#include <stdio.h>
#include <stdlib.h>

#ifndef scalar
#define scalar double
#endif

typedef int    *ivector;
typedef float  *fvector;
typedef double *dvector;
typedef scalar *vector;

typedef ivector *imatrix;
typedef fvector *fmatrix;
typedef dvector *dmatrix;
typedef vector *matrix;

ivector new_ivector(int);
void free_ivector(ivector);

fvector new_fvector(int);
void free_fvector(fvector);

dvector new_dvector(int);
void free_dvector(dvector);

vector new_vector(int);
void free_vector(vector);

imatrix new_imatrix(int, int);
void free_imatrix(imatrix);

fmatrix new_fmatrix(int, int);
void free_fmatrix(fmatrix);

dmatrix new_dmatrix(int, int);
void free_dmatrix(dmatrix);

matrix new_matrix(int, int);
void free_matrix(matrix);

scalar dotprod(int, vector, vector);

#ifdef QUADMATH_H
#include <quadmath.h>
typedef __float128 *vector128;
typedef vector128 *matrix128;
vector128 new_vector128(int);
void free_vector128(vector128);
matrix128 new_matrix128(int, int);
void free_matrix128(matrix128);
scalar dotprod128(int, vector128, vector128);
#endif

#endif
